// const logger = require('utils/logger');

const Widget = require("../models/widget");
const Template = require("../models/template");
const logger = require('../utils/logger'); 


const updateAndMapWidget = async (page, widgetId, position, startTime, endTime, templateId) => {
  try {
    let template;

    // Fetching template by templateId if it exists, else fallback to page name
    if (templateId) {
      logger.info(`Service | Fetching template by templateId: ${templateId}`);
      template = await Template.findOne({ _id: templateId });
    } else {
      logger.info(`Service | Fetching template for page: ${page}`);
      template = await Template.findOne({ name: page });
    }

    // If template is not found
    if (!template) {
      logger.error(`Template not found | Page: ${page}, Template ID: ${templateId || 'N/A'}`);
      return { status: 404, error: 'Template not found' };
    }

    logger.info(`Mapping widget | Page: ${page}, Widget ID: ${widgetId}, Position: ${position}`);

    // Find if the widget already exists in the template
    const widgetIndex = template.widgets.findIndex(w => w.widgetId.toString() === widgetId);

    // Update existing widget or add new one
    if (widgetIndex !== -1) {
      logger.info(`Widget found | Updating existing widget | Widget ID: ${widgetId}`);
      template.widgets[widgetIndex] = {
        ...template.widgets[widgetIndex],
        position,
        startTime,
        endTime
      };
    } else {
      logger.info(`Widget not found | Adding new widget | Widget ID: ${widgetId}`);
      template.widgets.push({
        widgetId,
        position,
        startTime,
        endTime
      });
    }

    // Save the updated template
    await template.save();
    logger.info(`Widget successfully saved | Widget ID: ${widgetId}, Page: ${page}, Template ID: ${templateId}`);

    return { status: 200, widgetId, page, templateId: template._id };

  } catch (error) {
    logger.error(`Error in Service updateAndMapWidget: ${error.message}`);
    throw error;
  }
};


const createTemplate = async (name, desc, platform, widgets, hashId) => {
  try {
    logger.info('Service: Creating new template in the database');

    const newTemplate = new Template({
      name,
      desc,
      platform,
      widgets,
      hashId,
    });

    // Save the template to the database
    await newTemplate.save();

    logger.info(`Template created successfully with id=${newTemplate._id}`);
    return { status: 201, message: 'Template created successfully', id: newTemplate._id };
  } catch (error) {
    logger.error(`Error saving template to database: ${error.message}`, { error });
    throw error;
  }
};


const getParticularTemplate = async (page, platform) => {
  try {
    let pipeline = [];
    pipeline.push({ $match: { name: page, platform: platform } });

    pipeline.push({ $unwind: { path: "$widgets" } });
    pipeline.push({
      $lookup: {
        localField: "widgets.widgetId",
        foreignField: "_id",
        from: "widgets",
        as: "widget",
      },
    });
    pipeline.push({ $unwind: { path: "$widget" } });

    const group = {
      $group: {
        _id: "$_id",
        name: { $first: "$name" },
        desc: { $first: "$desc" },
        hashId: { $first: "$hashId" },
        platform: { $first: "$platform" },

        widgets: {
          $addToSet: {
            widgetId: "$widget._id",
            title: "$widget.title",
            widget: "$widget.widget",
            enable: "$widget.enable",
            sectionHeadingText: "$widget.sectionHeadingText",
            apiUrl: "$widget.apiUrl",
            buttonData: "$widget.buttonData",
            list: "$widget.list",
            position: "$widgets.position",
            startTime: "$widgets.startTime",
            endTime: "$widgets.endTime",
          },
        },
      },
    };
    pipeline.push(group);
    const template = await Template.aggregate(pipeline);

    
    return template;
  } catch (error) {
    throw error;
  }
};



module.exports = {
  updateAndMapWidget,
  createTemplate,
  getParticularTemplate,
};